/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *
 *   Copyright (C) 2001-2006  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 *********************************************************************/

#include "constants.h"

#include "marriage.h"

#include "../../misc/translations.h"

/**
 ** constructor
 **
 ** @param	bridegroom	player with the marriage
 ** @param	bride		the partner
 **
 ** @return	-
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
GameplayAction::Marriage::Marriage(unsigned const bridegroom,
				     unsigned const bride) :
  GameplayAction(MARRIAGE, bridegroom),
  bride(bride)
{
} // Marriage::Marriage()

/**
 ** constructor
 **
 ** @param	line	line with the data
 **
 ** @return	action corresponding to the line
 **		(to be freed)
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 ** 
 ** @todo	old format
 ** @todo	new format
 **/
GameplayAction::Marriage::Marriage(string line) :
  GameplayAction(MARRIAGE, UINT_MAX),
  bride()
{
#ifndef OUTDATED
  // old format in the bug report:
  //   Played: Player 2: club ace
  if (string(line, 0, strlen("Marriage: ")) == "Marriage: ") {
    // ToDo

    return ;
  }
#endif

  // ToDo: new format

} // GameplayAction::Marriage(string line)

/**
 ** copy constructor
 **
 ** @param	marriage	action to copy
 **
 ** @return	-
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
GameplayAction::Marriage::Marriage(Marriage const& marriage) :
  GameplayAction(marriage),
  bride(marriage.bride)
{
} // Marriage::Marriage(Marriage marriage)

/**
 ** copy constructor
 **
 ** @param	marriage	action to copy
 **
 ** @return	this action
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
GameplayAction::Marriage::Marriage&
GameplayAction::Marriage::operator=(Marriage const& marriage)
{
  this->GameplayAction::operator=(marriage);
  this->bride = marriage.bride;
  return *this;
} // Marriage& Marriage::operator=(Marriage marriage)

/**
 ** destructor
 **
 ** @param	-
 **
 ** @return	-
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
GameplayAction::Marriage::~Marriage()
{
} // Marriage::~Marriage()

/**
 **
 ** writes the action in the output stream
 **
 ** @param	ostr	output stream
 **
 ** @return	-
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
void
GameplayAction::Marriage::write(ostream& ostr) const
{
  ostr << this->type;
  ostr << " = " << this->player << ", " << this->bride;

  return ;
} // void write(ostream& ostr) const

/**
 ** -> result
 **
 ** @param	-
 **
 ** @return	a translation representing the data
 ** 
 ** @author	Diether Knof
 **
 ** @version	0.7.1
 **/
Translator::Translation
GameplayAction::Marriage::data_translation() const
{
  return ::translation("%uunsigned%", this->bride);
} // virtual Translator::Translation GameplayAction::Marriage::data_translation() const
