/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *
 *   Copyright (C) 2001-2006  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 *********************************************************************/

#include "constants.h"

#include "trick_full.h"

#include "../../misc/translations.h"

/**
 ** constructor
 **
 ** @param	trick	full trick
 **
 ** @return	-
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
GameplayAction::TrickFull::TrickFull(Trick const& trick) :
  GameplayAction(TRICK_FULL, UINT_MAX),
  trick(trick)
{
} // TrickFull::TrickFull()

/**
 ** constructor
 **
 ** @param	line	line with the data
 ** @param	istr	input stream (behind the line)
 **
 ** @return	action corresponding to the line
 **		(to be freed)
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 ** 
 ** @todo	old format
 ** @todo	new format
 **/
GameplayAction::TrickFull::TrickFull(string const& line,
				     istream& istr) :
  GameplayAction(TRICK_FULL, UINT_MAX),
  trick()
{
#ifndef OUTDATED
  // old format in the bug report:
  //   Played: Player 2: club ace
  if (string(line, 0, strlen("Trick full: ")) == "Trick full: ") {
    this->trick = Trick(istr);

    return ;
  }
#endif

  // ToDo: new format

} // GameplayAction::TrickFull(string line, istream& istr)

/**
 ** copy constructor
 **
 ** @param	trick_full	action to copy
 **
 ** @return	-
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
GameplayAction::TrickFull::TrickFull(TrickFull const& trick_full) :
  GameplayAction(trick_full),
  trick(trick_full.trick)
{
} // TrickFull::TrickFull(TrickFull trick_full)

/**
 ** copy operator
 **
 ** @param	trick_full	action to copy
 **
 ** @return	this action
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
GameplayAction::TrickFull::TrickFull&
GameplayAction::TrickFull::operator=(TrickFull const& trick_full)
{
  this->GameplayAction::operator=(trick_full);
  this->trick = trick_full.trick;
  return *this;
} // TrickFull& TrickFull::operator=(TrickFull trick_full)

/**
 ** destructor
 **
 ** @param	-
 **
 ** @return	-
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
GameplayAction::TrickFull::~TrickFull()
{
} // TrickFull::~TrickFull()

/**
 ** writes the action in the output stream
 **
 ** @param	ostr	output stream
 **
 ** @return	-
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
void
GameplayAction::TrickFull::write(ostream& ostr) const
{
  ostr << this->type << " = "
    << this->trick.startplayerno();
  for (unsigned c = 0; c < this->trick.actcardno(); ++c)
    ostr << ", " << this->trick.card(c);

  // cannot use 'Trick::isfull()' because that needs a corresponding game/party
  if (this->trick.winnerplayerno() != UINT_MAX)
    ostr << ", " << this->trick.winnerplayerno();

  return ;
} // void write(ostream& ostr) const

/**
 ** -> result
 **
 ** @param	-
 **
 ** @return	a translation representing the data
 ** 
 ** @author	Diether Knof
 **
 ** @version	0.7.1
 **/
Translator::Translation
GameplayAction::TrickFull::data_translation() const
{
  if (this->trick.actcardno() == 0)
    return ::translation("");

  Translator::Translation translation(::translator);

  if (this->trick.winnerplayerno() != UINT_MAX)
    translation
      += ::translation("%uunsigned%", this->trick.winnerplayerno()) + " -- ";

  translation += ::translation(this->trick.card(0));
  for (unsigned c = 0; c < this->trick.actcardno(); ++c)
    translation += ", " + ::translation(this->trick.card(c));

  return translation;
} // virtual Translator::Translation GameplayAction::TrickFull::data_translation() const
